package com.halo.halojuhesdk.core.base.net;

import android.text.TextUtils;

import com.halo.halojuhesdk.core.base.ReqType;
import com.halo.halojuhesdk.core.base.utils.Utils;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;

import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;

/**
 * Created by dream on 2016/8/11.
 */

public class HlConnectionManager {

    private ReqType mReqType;
    private URL mUrl;
    private boolean isGzipRequest = false;
    private boolean isHttps = false;

    /**
     * HTTP链接超时时间
     */
    private int http_timeout = 180000;
    /**
     * 请求获取数据超时时间
     */
    private int socket_timeout = 180000;

    public HlConnectionManager(ReqType reqType, String url) throws Exception {
        this.mReqType = reqType;
        if (!TextUtils.isEmpty(url)) {
            String str_http = url.substring(0, url.indexOf(":"));
            if (str_http.equals("https")) {
                isHttps = true;
            }
        } else {
            throw new Exception("URL is Empty");
        }
        mUrl = new URL(url);
    }

    public void setGzipRequest(boolean isGzipRequest) {
        this.isGzipRequest = isGzipRequest;
    }

    public void setTimeOut(int http_timeout, int socket_timeout) {
        this.http_timeout = http_timeout;
        this.socket_timeout = socket_timeout;
    }

    public void openConnection(AbstractNetTask abstractNetTask) throws Exception {
        if (isHttps) {
            dealWithHttpsConnection(abstractNetTask);
        } else {
            dealWithHttpConnection(abstractNetTask);
        }
    }



    private void dealWithHttpConnection(AbstractNetTask abstractNetTask) throws Exception {
        HttpURLConnection httpURLConnection = null;
        InputStream inputStream = null;
        try {
            httpURLConnection = getHttpURLConnection();
            if (isGzipRequest) {
                httpURLConnection.addRequestProperty("Accept-Encoding", "gzip,deflate");
            }else{
                httpURLConnection.addRequestProperty("Accept-Encoding","");
            }
            abstractNetTask.onAddHeaders(httpURLConnection);
            switch (mReqType) {
                case Get:
                    httpURLConnection.setRequestMethod("GET");
                    break;
                case Post:
                    httpURLConnection.setRequestMethod("POST");
                    httpURLConnection.setDoOutput(true);
                    OutputStream outputStream = null;
                    try {
                        outputStream = httpURLConnection.getOutputStream();
                        abstractNetTask.getEntity(outputStream);
                    } catch (Exception e) {
                        e.printStackTrace();
                    } finally {
                        try {
                            if (outputStream != null) {
                                outputStream.close();
                            }
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                    }
                    break;
            }
            int responseCode = httpURLConnection.getResponseCode();
            String content;
            inputStream = httpURLConnection.getInputStream();
            String contentEncoding = httpURLConnection.getContentEncoding();
            content = Utils.parseInputString(Utils.decodeInputStream(inputStream, contentEncoding));
            if (TextUtils.isEmpty(content)) {
                content = httpURLConnection.getResponseMessage();
            }
            abstractNetTask.onRecive(responseCode, content);
        } finally {
            try {
                if (inputStream != null) {
                    inputStream.close();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }

            try {
                if (httpURLConnection != null) {
                    httpURLConnection.disconnect();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    private HttpURLConnection getHttpURLConnection() throws IOException {
        HttpURLConnection httpURLConnection;
        httpURLConnection = (HttpURLConnection) mUrl.openConnection();
        httpURLConnection.setConnectTimeout(socket_timeout);
        httpURLConnection.setReadTimeout(http_timeout);
        httpURLConnection.setInstanceFollowRedirects(true);
        httpURLConnection.addRequestProperty("Accept-Charset", "UTF-8;");
        return httpURLConnection;
    }

    private void dealWithHttpsConnection(AbstractNetTask abstractNetTask) throws Exception {
        HttpsURLConnection httpsURLConnection = null;
        InputStream inputStream = null;
        try {
            httpsURLConnection = getHttpsURLConnection();
            if (isGzipRequest) {
                httpsURLConnection.addRequestProperty("Accept-Encoding", "gzip,deflate");
            } else{
                httpsURLConnection.addRequestProperty("Accept-Encoding","");
            }
            abstractNetTask.onAddHeaders(httpsURLConnection);
            switch (mReqType) {
            case Get:
                httpsURLConnection.setRequestMethod("GET");
                break;
            case Post:
                httpsURLConnection.setRequestMethod("POST");
                httpsURLConnection.setDoOutput(true);
                OutputStream outputStream = null;
                try {
                    outputStream = httpsURLConnection.getOutputStream();
                    abstractNetTask.getEntity(outputStream);
                } catch (Exception e) {
                    e.printStackTrace();
                } finally {
                    try {
                        if (outputStream != null) {
                            outputStream.close();
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                    }

                }
                break;
        }
        int responseCode = httpsURLConnection.getResponseCode();
        String content;
        inputStream = httpsURLConnection.getInputStream();
        String contentEncoding = httpsURLConnection.getContentEncoding();
        content = Utils.parseInputString(Utils.decodeInputStream(inputStream, contentEncoding));
        if (TextUtils.isEmpty(content)) {
            content = httpsURLConnection.getResponseMessage();
        }
        abstractNetTask.onRecive(responseCode, content);
    } finally {
        try {
            if (inputStream != null) {
                inputStream.close();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

            try {
                if (httpsURLConnection != null) {
                    httpsURLConnection.disconnect();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }


        }
    }

    private HttpsURLConnection getHttpsURLConnection() throws IOException, NoSuchAlgorithmException, KeyManagementException {
        HttpsURLConnection httpsURLConnection;
        httpsURLConnection = (HttpsURLConnection) mUrl.openConnection();
        httpsURLConnection.setConnectTimeout(socket_timeout);
        httpsURLConnection.setReadTimeout(http_timeout);
        TrustManager[] tm = {new MyX509TrustManager()};
        SSLContext sslContext = SSLContext.getInstance("SSL");
        sslContext.init(null, tm, null);
        httpsURLConnection.setSSLSocketFactory(sslContext.getSocketFactory());
        httpsURLConnection.setHostnameVerifier(new TrustAnyHostnameVerifier());
        httpsURLConnection.setInstanceFollowRedirects(true);
        httpsURLConnection.addRequestProperty("Accept-Charset", "UTF-8;");
        return httpsURLConnection;
    }
}
