/*
 * Copyright (C) 2014 Huanju Inc. All rights reserved.
 */
package com.halo.halojuhesdk.core.base.utils;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.Writer;
import java.lang.Thread.UncaughtExceptionHandler;

/**
 * Hunanju 未捕获异常处理类
 * 
 * 整体思路是捕获异常之后抛出异常
 * 
 * @author Wang mingjun
 * @since 2015-10-21
 */
public class HjExceptionHandler implements UncaughtExceptionHandler {
	// private static final String TAG = "HjExceptionHandler";
    // 缓存ExceptionHandler
    private static UncaughtExceptionHandler mExceptionHandler;

    // // 用于格式化日期,作为日志文件名的一部分
    // private DateFormat formatter = new
    // SimpleDateFormat("yyyy-MM-dd-HH-mm-ss");
    /*
     * 获取欢聚相关未捕获异常
     * 
     * 由于有些异常可能是无网络情况下出现的因此需要考虑无网络状态兼容
     * 
     * @see
     * java.lang.Thread.UncaughtExceptionHandler#uncaughtException(java.lang
     * .Thread, java.lang.Throwable)
     */
    @Override
    public void uncaughtException(Thread thread, Throwable ex) {
        String threadLog = readCrashInfo(ex);
        if (threadLog.contains("com.huanju")) {
//			 HjInfoGatheringManager.getInstance(HjSearchClient.getContext()).crashAdd();
//			 new HjReportExceptionProcessor(HjSearchClient.getContext(),
//			 threadLog).process();
        }
        if (mExceptionHandler != null) {
            mExceptionHandler.uncaughtException(thread, ex);
        }

    }

    /**
     * @description 注册异常捕获
     * @see
     * @version
     * @author Wang mingjun
     * @since 2015-10-21
     * @tags 需要在主线程调用
     */
    public static void initExceptionHandler() {
        mExceptionHandler = Thread.getDefaultUncaughtExceptionHandler();
        Thread.setDefaultUncaughtExceptionHandler(new HjExceptionHandler());
    }

    private String readCrashInfo(Throwable ex) {

        StringBuffer sb = new StringBuffer();
        Writer writer = new StringWriter();
        PrintWriter printWriter = new PrintWriter(writer);
        ex.printStackTrace(printWriter);
        Throwable cause = ex.getCause();
        while (cause != null) {
            cause.printStackTrace(printWriter);
            cause = cause.getCause();
        }
        printWriter.close();
        String result = writer.toString();
        sb.append(result);
        return sb.toString();
    }
}
